import "SDRPlugins.SmaugPlugin";
import "Turbine.Gameplay";
import "Turbine.UI";
import "Turbine.UI.Lotro";

SVPUnloader = Turbine.UI.Control();

SVPUnloader.Update = function( sender, args )
	SVPUnloader:SetWantsUpdates( false );
	Turbine.PluginManager.UnloadScriptState("SVPReloader");
end

SVPUnloader:SetWantsUpdates( true );

-- safely add a callback without overwriting any existing ones
function AddCallback(object, event, callback)
    if object[event] == nil then
        object[event] = callback;
    else
        if type(object[event]) == "table" then
            table.insert(object[event], callback);
        else
            object[event] = {object[event], callback};
        end
    end
end

-- safely remove a callback without clobbering any extras
function RemoveCallback(object, event, callback)
    if object[event] == callback then
        object[event] = nil;
    else
        if type(object[event]) == "table" then
            local size = table.getn(object[event]);
            for i = 1, size do
                if object[event][i] == callback then
                    table.remove(object[event], i);
                    break;
                end
            end
        end
    end
end

SVPMain = class();

function SVPMain:Constructor()
	my_player = Turbine.Gameplay.LocalPlayer:GetInstance();
	party = my_player:GetParty();
	if party ~= nil then party:GetMember(1):GetName() end; -- Apparently this will prevent the plugin from crashing the next time GetParty() is called
	import "SDRPlugins.SmaugPlugin.ValveController";
	self.EffectTracker = EffectTracker();
	import "SDRPlugins.SmaugPlugin.MainWindow";
end

function SVPMain:RemoveCallbacks()
	self.EffectTracker:RemoveEffectCallbacks();
	self.EffectTracker:RemovePartyCallbacks();
end

main = SVPMain();

Turbine.Plugin.Load = function(self, sender, args)
	Turbine.Shell.WriteLine(self:GetName() .. " v" .. self:GetVersion() .. " by " .. self:GetAuthor() .. " loaded!");
	Version = self:GetVersion();
end

Turbine.Plugin.Unload = function(self, sender, args)
	main:RemoveCallbacks();
	RemoveCallback(Turbine.Chat, "Received", startFightTimer);
	saveState();
	self.EffectTracker = nil;
	SVPUnloader.Update = nil;
	SVPUnloader = nil;
end

-- create a new command line command for PDT options
SVPCommand = Turbine.ShellCommand();

-- handle the chat commands
function SVPCommand:Execute( name, shellCommand )
	if shellCommand == "reload" then
		Turbine.Shell.WriteLine("Force SmaugPlugin reload.");
	else 
		SVPCommand:GetHelp();
	end
end

-- The Help text that outputs when /SmaugPlugin is typed
help = "SmaugPlugin has the following command:\n" ..
	"reload - reloads the plugin (this can fix lag issues).";

function SVPCommand:GetHelp()
	Turbine.Shell.WriteLine( help );
end

-- add the command to the shell
Turbine.Shell.AddCommand( "smaugplugin", SVPCommand );