import "Turbine";
import "Turbine.Gameplay";
import "Turbine.UI";
import "Turbine.UI.Lotro";

-- Credit to Cee for following code:

-- General functions
function createTarget()
	local t = Turbine.UI.Label();
	t.MaximumMorale = 0;
	t.CurrentMorale = 1;
	t.Name = "Belegnor";
	t.PercentageLabel = Turbine.UI.Label();
	t.PercentageLabel.ParentTarget = t;
	return t;
end

function getPercentage(target)
	local text = "-";
	if ( target.MaximumMorale ~= nil and target.MaximumMorale ~= 0 ) then
		text = round(target.CurrentMorale / target.MaximumMorale * 100, 2) .. "%";
	end
	return text;
end

function round(num, idp)
  local mult = 10^(idp or 0);
  return math.floor(num * mult + 0.5) / mult;
end

function styleLabel(lbl, x, y, w, h, font, color, align, text)
	lbl:SetPosition(x,y);
	lbl:SetSize(w,h);
	lbl:SetFont(font);
	lbl:SetText(text);
	lbl:SetTextAlignment(align);
	lbl:SetForeColor(color);
	lbl:SetMouseVisible(false);
	lbl:SetZOrder(3);
end

-- Window Dragging functions
is_dragging = false;
function startDrag(sender, args)
	if ( args.Button ~= Turbine.UI.MouseButton.Left ) then return; end
	startX = args.X;
	startY = args.Y;
	is_dragging = true;
end

function endDrag(sender, args)
	if ( is_dragging ) then
		main_window:SetLeft(main_window:GetLeft() + (args.X - startX));
		main_window:SetTop(main_window:GetTop() + (args.Y - startY));
		is_dragging = false;
	end
end

function doDrag(sender, args)
	if ( is_dragging ) then
		main_window:SetLeft(main_window:GetLeft() + (args.X - startX));
		main_window:SetTop(main_window:GetTop() + (args.Y - startY));
	end
end

function attachDragListener(target)
	target.MouseDown = startDrag;
	target.MouseUp = endDrag;
	target.MouseMove = doDrag;
end

-- Persistance functions
function saveState()
	local obj = Turbine.Object();
	obj.window_y = main_window:GetTop();
	obj.window_x = main_window:GetLeft();
	obj.startTime = gameTimeAtStart;
	obj.estSmoke = estSmokeInCaves;
	obj.lastMoraleThreshold = belegnor.lastMoraleThreshold;
	obj.northTopSteps = northTop.steps;
	obj.northTopNextOpen = northTop.nextOpen;
	obj.northBottomSteps = northBottom.steps;
	obj.northBottomNextOpen = northBottom.nextOpen;
	obj.southTopSteps = southTop.steps;
	obj.southTopNextOpen = southTop.nextOpen;
	obj.southBottomSteps = southBottom.steps;
	obj.southBottomNextOpen = southBottom.nextOpen;
	Turbine.PluginData.Save(Turbine.DataScope.Character, "SmaugPlugin", obj, nil);
end

function loadState()
	local persist = Turbine.PluginData.Load(Turbine.DataScope.Character, "SmaugPlugin", nil);
	if persist ~= nil then
		main_window:SetTop(persist.window_y);
		main_window:SetLeft(persist.window_x);
		gameTimeAtStart = persist.startTime;
		estSmokeInCaves = persist.estSmoke;
		belegnor.lastMoraleThreshold = persist.lastMoraleThreshold;
		northTop.steps = persist.northTopSteps;
		northTop.nextOpen = persist.northTopNextOpen;
		northBottom.steps = persist.northBottomSteps;
		northBottom.nextOpen = persist.northBottomNextOpen;
		southTop.steps = persist.southTopSteps;
		southTop.nextOpen = persist.southTopNextOpen;
		southBottom.steps = persist.southBottomSteps;
		southBottom.nextOpen = persist.southBottomNextOpen;
	end
end

-- Window functions - created by Cee, customized by Alvirhelm
function createMainWindow()
	-- main window
	main_window = Turbine.UI.Window();
	main_window:SetPosition(100,100);
	main_window:SetSize(380,60);
	attachDragListener(main_window);

	-- Fight Timer
	fightTimerLabel = Turbine.UI.Label();
	styleLabel(fightTimerLabel, 0, 0, 190, 15, Turbine.UI.Lotro.Font.TrajanPro15, Turbine.UI.Color(0,0,0), Turbine.UI.ContentAlignment.MiddleCenter, "Smaug Timer: ");
	fightTimerLabel:SetBackColor(Turbine.UI.Color(0,1,0));

	fightTimer = Turbine.UI.Label();
	styleLabel(fightTimer, 190, 0, 190, 15, Turbine.UI.Lotro.Font.TrajanPro15, Turbine.UI.Color(0,0,0), Turbine.UI.ContentAlignment.MiddleCenter, "25:00");
	fightTimer:SetBackColor(Turbine.UI.Color(0,1,0));

	fightTimerLabel:SetParent(main_window);
	fightTimer:SetParent(main_window);

	-- Valve labels	
	southeastValveLabel = Turbine.UI.Label();
	southwestValveLabel = Turbine.UI.Label();
	northeastValveLabel = Turbine.UI.Label();
	northwestValveLabel = Turbine.UI.Label();

	southeastValveLabel:SetBackColor(Turbine.UI.Color(0,0,0));
	southwestValveLabel:SetBackColor(Turbine.UI.Color(0,0,0));
	northeastValveLabel:SetBackColor(Turbine.UI.Color(245/255,223/255,146/255));
	northwestValveLabel:SetBackColor(Turbine.UI.Color(245/255,223/255,146/255));

	styleLabel(southeastValveLabel, 0, 15, 105, 15, Turbine.UI.Lotro.Font.TrajanPro15, Turbine.UI.Color(245/255,223/255,146/255), Turbine.UI.ContentAlignment.MiddleCenter, "South Bottom");
	styleLabel(southwestValveLabel, 105, 15, 85, 15, Turbine.UI.Lotro.Font.TrajanPro15, Turbine.UI.Color(245/255,223/255,146/255), Turbine.UI.ContentAlignment.MiddleCenter, "South Top");
	styleLabel(northeastValveLabel, 190, 15, 105, 15, Turbine.UI.Lotro.Font.TrajanPro15, Turbine.UI.Color(0,0,0), Turbine.UI.ContentAlignment.MiddleCenter, "North Bottom");
	styleLabel(northwestValveLabel, 295, 15, 85, 15, Turbine.UI.Lotro.Font.TrajanPro15, Turbine.UI.Color(0,0,0), Turbine.UI.ContentAlignment.MiddleCenter, "North Top");
	
	southeastValveLabel:SetParent(main_window);
	southwestValveLabel:SetParent(main_window);
	northeastValveLabel:SetParent(main_window);
	northwestValveLabel:SetParent(main_window);

	styleLabel(southBottom.label, 0, 30, 105, 15, Turbine.UI.Lotro.Font.TrajanPro15, Turbine.UI.Color(0,0,0), Turbine.UI.ContentAlignment.MiddleCenter, "5 (30)");
	styleLabel(southTop.label, 105, 30, 85, 15, Turbine.UI.Lotro.Font.TrajanPro15, Turbine.UI.Color(0,0,0), Turbine.UI.ContentAlignment.MiddleCenter, "5 (30)");
	styleLabel(northBottom.label, 190, 30, 105, 15, Turbine.UI.Lotro.Font.TrajanPro15, Turbine.UI.Color(0,0,0), Turbine.UI.ContentAlignment.MiddleCenter, "5 (30)");
	styleLabel(northTop.label, 295, 30, 85, 15, Turbine.UI.Lotro.Font.TrajanPro15, Turbine.UI.Color(0,0,0), Turbine.UI.ContentAlignment.MiddleCenter, "5 (30)");
	
	southBottom.label:SetBackColor(Turbine.UI.Color(1,0,0));
	southTop.label:SetBackColor(Turbine.UI.Color(1,0,0));
	northBottom.label:SetBackColor(Turbine.UI.Color(1,0,0));
	northTop.label:SetBackColor(Turbine.UI.Color(1,0,0));

	southBottom.label:SetParent(main_window);
	southTop.label:SetParent(main_window);
	northBottom.label:SetParent(main_window);
	northTop.label:SetParent(main_window);

	-- Boss health percent
	styleLabel(belegnor, 0, 45, 145, 15, Turbine.UI.Lotro.Font.TrajanPro15, Turbine.UI.Color(245/255,223/255,146/255), Turbine.UI.ContentAlignment.MiddleCenter, "Belegnor Morale %: ");
	belegnor:SetBackColor(Turbine.UI.Color(0,0,0));

	styleLabel(belegnor.PercentageLabel, 145, 45, 55, 15, Turbine.UI.Lotro.Font.TrajanPro15, Turbine.UI.Color(245/255,223/255,146/255), Turbine.UI.ContentAlignment.MiddleCenter, "100.00%");
	belegnor.PercentageLabel:SetBackColor(Turbine.UI.Color(0,0,0));

	belegnor:SetParent(main_window);
	belegnor.PercentageLabel:SetParent(main_window);

	-- smoke in cave labels
	smokeLabel = Turbine.UI.Label();
	styleLabel(smokeLabel, 200, 45, 150, 15, Turbine.UI.Lotro.Font.TrajanPro15, Turbine.UI.Color(0,0,0), Turbine.UI.ContentAlignment.MiddleCenter, "Est. Smoke In Caves: ");
	smokeLabel:SetBackColor(Turbine.UI.Color(245/255,223/255,146/255));

	smoke = Turbine.UI.Label();
	styleLabel(smoke, 350, 45, 30, 15, Turbine.UI.Lotro.Font.TrajanPro15, Turbine.UI.Color(0,0,0), Turbine.UI.ContentAlignment.MiddleCenter, "0");
	smoke:SetBackColor(Turbine.UI.Color(1,1,0));

	smokeLabel:SetParent(main_window);
	smoke:SetParent(main_window);
end

-- Tracking functions
function targetChanged(sender, args)
	local t = sender:GetTarget();
	if ( t ~= nil ) then
		if ( t:GetName() == belegnor.Name ) then
			onMoraleChanged(t, nil);
			t.MoraleChanged = onMoraleChanged;
		end
	end
end

-- moraleFlasher flashes the morale percent when the boss passes a 5% morale threshold
moraleFlasherControl = Turbine.UI.Control();
moraleFlasherControl.time = 0; -- the last time (in game seconds) that the boss passes a morale threshold

moraleFlasherControl.Update = function(sender, args)
	local remTime = Turbine.Engine:GetGameTime();
	local flash = remTime - math.floor(remTime);
	flash = 0.5 * math.sin(flash * 2 * 3.14159) + 0.5;
	belegnor.PercentageLabel:SetBackColor(Turbine.UI.Color(flash, 0, 0));

	if moraleFlasherControl.time + 3 < remTime then
		belegnor.PercentageLabel:SetBackColor(Turbine.UI.Color(0,0,0));
		moraleFlasherControl:SetWantsUpdates(false);
	end
end

function onMoraleChanged(sender, args)
	if ( sender:GetName() == belegnor.Name ) then
		belegnor.MaximumMorale = sender:GetMaxMorale();
		belegnor.CurrentMorale = sender:GetMorale();
		if ( belegnor.MaximumMorale < belegnor.CurrentMorale ) then belegnor.MaximumMorale = belegnor.CurrentMorale; end

		if ( belegnor.MaximumMorale ~= nil and belegnor.MaximumMorale ~= 0 ) then
			local moralePercent = round(belegnor.CurrentMorale / belegnor.MaximumMorale * 100, 2);
			if moralePercent % 1 == 0 then
				belegnor.PercentageLabel:SetText(moralePercent .. ".00%");
			else belegnor.PercentageLabel:SetText(moralePercent .. "%");
			end
			if belegnor.lastMoraleThreshold - moralePercent >= 4.99 then
				belegnor.lastMoraleThreshold = belegnor.lastMoraleThreshold - 5;
				local closingTime = Turbine.Engine:GetGameTime() - 7;
				moraleFlasherControl.time = closingTime + 7;
				moraleFlasherControl:SetWantsUpdates(true);
				Turbine.Shell.WriteLine(closingTime .. " - " .. belegnor.lastMoraleThreshold);

				forceValvesOpen(northTop, closingTime);
				forceValvesOpen(northBottom, closingTime);
				forceValvesOpen(southTop, closingTime);
				forceValvesOpen(southBottom, closingTime);

				-- track est. smoke in caves (BEFORE VALVES OPEN)
				smoke:SetText(estSmokeInCaves);
			end
		end
	end
end

function attachTargetListeners()
	--local myTarget = my_player:GetTarget();
	--if ( myTarget ~= nil ) then targetChanged(my_player, nil); end
	--my_player.TargetChanged = targetChanged;

	if ( party ~= nil ) then
		party:GetMember(1):GetName(); -- TURBINE BUG WORKAROUND: Not accessing GetMember will crash the client on next GetParty call...
		for idx=1,party:GetAssistTargetCount() do
			local rat = party:GetAssistTarget(idx);
			--if ( rat:GetName() ~= my_player:GetName() ) then
				local ratTarget = rat:GetTarget();
				if ( ratTarget ~= nil ) then targetChanged(rat, nil); end
				rat.TargetChanged = targetChanged;
			--end
		end
	end
end

-- monitors chat for different things
function startFightTimer(sender, args)
	-- starts fight timer
	if args.ChatType == Turbine.ChatType.Quest then
		if(args.Message:find("The battle has begun", 1, true) ~= nil) then
			gameTimeAtStart = round(Turbine.Engine:GetGameTime(), 1);
			northTop.steps = 5;
			northBottom.steps = 5;
			southTop.steps = 5;
			southBottom.steps = 5;
			belegnor.lastMoraleThreshold = 100;
			estSmokeInCaves = 0;
			smoke:SetText(estSmokeInCaves);
			timerControl:SetWantsUpdates(true);
		end

	-- pops valves if needed during fight tender stages
	elseif args.ChatType == Turbine.ChatType.Unfiltered then
		if(args.Message:find("As the grim goes dark", 1, true) ~= nil) then
			if belegnor.lastMoraleThreshold == 80 or belegnor.lastMoraleThreshold == 55 or belegnor.lastMoraleThreshold == 30 then
				belegnor.lastMoraleThreshold = belegnor.lastMoraleThreshold - 5;
				local tenderTime = Turbine.Engine:GetGameTime()-5;
				Turbine.Shell.WriteLine(tenderTime .. " Grim morale threshold: " .. belegnor.lastMoraleThreshold);
	
				forceValvesOpen(northTop, tenderTime);
				forceValvesOpen(northBottom, tenderTime);
				forceValvesOpen(southTop, tenderTime);
				forceValvesOpen(southBottom, tenderTime);

				smoke:SetText(estSmokeInCaves);
			end
		end
	end
end

-- Global vars
main_window = nil;
belegnor = createTarget();
belegnor.lastMoraleThreshold = 100;
belegnor.lastMoraleChange = 0;
fightLength = 1500;
gameTimeAtStart = 0;
estSmokeInCaves = 0;

-- Load plugin
createMainWindow();
loadState();
AddCallback(Turbine.Chat, "Received", startFightTimer);
attachTargetListeners();

-- Run
main_window:SetVisible(true);

-- timer controller monitors the game time and updates the fight timer
timerControl = Turbine.UI.Control();

timerControl.Update = function(sender, args)
	local gameTime = round(Turbine.Engine:GetGameTime(), 1);

	-- update fight timer
	local fightDuration = gameTime - gameTimeAtStart;
	if fightDuration > 1500 then
		gameTimeAtStart = 0;
		timerControl:SetWantsUpdates(false);
		return;
	end
	local timeRemaining = round((fightLength - fightDuration), 0);
	local seconds = (timeRemaining % 60);
	local mins = (timeRemaining - seconds) / 60;
	if seconds == 60 then
		seconds = 0;
	end
	if seconds > 9 then
		fightTimer:SetText(mins .. ":" .. seconds);
	else fightTimer:SetText(mins .. ":0" .. seconds);
	end

	-- open valves if needed
	if northTop.nextOpen < gameTime then
		changeValveSteps(northTop, 1, gameTime);
	else setValveLabel(northTop, gameTime);
	end
	if northBottom.nextOpen < gameTime then
		changeValveSteps(northBottom, 1, gameTime);
	else setValveLabel(northBottom, gameTime);
	end
	if southTop.nextOpen < gameTime then
		changeValveSteps(southTop, 1, gameTime);
	else setValveLabel(southTop, gameTime);
	end
	if southBottom.nextOpen < gameTime then
		changeValveSteps(southBottom, 1, gameTime);
	else setValveLabel(southBottom, gameTime);
	end

	if fightDuration / 60 < 8.5 then
		fightTimerLabel:SetBackColor(Turbine.UI.Color(0,1,0));
	elseif fightDuration / 60 >= 8.5 and fightDuration / 60 < 14.75 then
		fightTimerLabel:SetBackColor(Turbine.UI.Color(1,1,0));
		if belegnor.lastMoraleThreshold > 75 then
			fightTimer:SetBackColor(Turbine.UI.Color(1,0,0));
		else fightTimer:SetBackColor(Turbine.UI.Color(0,1,0));
		end
	elseif fightDuration / 60 >= 14.75 and fightDuration / 60 < 20.5 then
		fightTimerLabel:SetBackColor(Turbine.UI.Color(1,.5,0));
		if belegnor.lastMoraleThreshold > 50 then
			fightTimer:SetBackColor(Turbine.UI.Color(1,0,0));
		else fightTimer:SetBackColor(Turbine.UI.Color(0,1,0));
		end
	elseif fightDuration / 60 < 24 then
		fightTimerLabel:SetBackColor(Turbine.UI.Color(1,0,0));
		if belegnor.lastMoraleThreshold > 25 then
			fightTimer:SetBackColor(Turbine.UI.Color(1,0,0));
		else fightTimer:SetBackColor(Turbine.UI.Color(0,1,0));
		end
	else
		local remTime = Turbine.Engine:GetGameTime();
		local flash = remTime - math.floor(remTime);
		flash = 0.5 * math.sin(flash * 2 * 3.14159) + 0.5;
		fightTimerLabel:SetBackColor(Turbine.UI.Color(flash, 0, 0));
	end

	local possSmoke = estSmokeInCaves + (20 * ((belegnor.lastMoraleThreshold / 5) - 1));
	if possSmoke < 200 then
		smoke:SetBackColor(Turbine.UI.Color(0,1,0));
	end
end

if gameTimeAtStart ~= 0 then
	timerControl:SetWantsUpdates(true);
	smoke:SetText(estSmokeInCaves);
end