-- Variables

-- valves
northTop = {};
northBottom = {};
southTop = {};
southBottom = {};

-- Time to go from fully closed (0) to 1 step open (1)
TIME_FOR_FIRST_STEP = 90;
-- Time to go from 1 (or higher) steps to the next step up
TIME_FOR_OTHER_STEPS = 30;

-- Valve steps - they start at 5 at the beginning of the fight
northTop.steps = 5;
northBottom.steps = 5;
southTop.steps = 5;
southBottom.steps = 5;

-- timers for when valve opens to next step
northTop.nextOpen = 0;
northBottom.nextOpen = 0;
southTop.nextOpen = 0;
southBottom.nextOpen = 0;

-- UI Labels for valve numbers
northTop.label = Turbine.UI.Label();
northBottom.label = Turbine.UI.Label();
southTop.label = Turbine.UI.Label();
southBottom.label = Turbine.UI.Label();

-- 2nd most recent close
northTop.lastClosed1 = 0;
northTop.lastPosition1 = 0;
northBottom.lastClosed1 = 0;
northBottom.lastPosition1 = 0;
southTop.lastClosed1 = 0;
southTop.lastPosition1 = 0;
southBottom.lastClosed1 = 0;
southBottom.lastPosition1 = 0;

-- most recent close
northTop.lastClosed2 = 0;
northTop.lastPosition2 = 0;
northBottom.lastClosed2 = 0;
northBottom.lastPosition2 = 0;
southTop.lastClosed2 = 0;
southTop.lastPosition2 = 0;
southBottom.lastClosed2 = 0;
southBottom.lastPosition2 = 0;

-- functions

function changeValveSteps(valve, steps, gameTime)
	-- keep track of previous steps
	if steps < 0 then
		valve.lastClosed1 = valve.lastClosed2;
		valve.lastClosed2 = gameTime;
		
		valve.lastPosition1 = valve.lastPosition2;
		valve.lastPosition2 = valve.steps;
	end	

	-- change valve step by steps
	valve.steps = valve.steps + steps;
	if valve.steps < 0 then
		valve.steps = 0; -- makes sure valve isn't lower than 0
	elseif valve.steps > 5 then
		valve.steps = 5; -- makes sure valve isn't higher than 5
	end

	-- sets next open time
	if valve.steps > 0 then -- valve opens in 30 seconds if valve is not fully closed
		valve.nextOpen = gameTime + TIME_FOR_OTHER_STEPS;
	else valve.nextOpen = gameTime + TIME_FOR_FIRST_STEP; -- valve opens in 90 seconds if fully closed (0)
	end
	setValveLabel(valve, gameTime);
end

function setValveLabel(valve, gameTime)
	local remTime = Turbine.Engine:GetGameTime();
	local flash = remTime - math.floor(remTime);
	flash = ((0.5 * math.sin(flash * 2 * 3.14159) + 0.5) / 2) + .5;

	local valveTimer = round(valve.nextOpen - gameTime, 0);
	valve.label:SetText(valve.steps .. " (" .. valveTimer .. ")"); -- set valve step on UI
	if valve.steps == 0 then
		valve.label:SetBackColor(Turbine.UI.Color(0,1,0));
		if valveTimer < 10 then
			valve.label:SetBackColor(Turbine.UI.Color(0,flash,0));
		end
	elseif valve.steps == 1 or valve.steps == 2 then
		valve.label:SetBackColor(Turbine.UI.Color(1,1,0));
		if valveTimer < 10 then
			valve.label:SetBackColor(Turbine.UI.Color(flash,flash,0));
		end
	elseif valve.steps == 3 or valve.steps == 4 then
		valve.label:SetBackColor(Turbine.UI.Color(1,.5,0));
		if valveTimer < 10 then
			valve.label:SetBackColor(Turbine.UI.Color(flash,flash - .5,0));
		end
	else
		valve.label:SetBackColor(Turbine.UI.Color(1,0,0));
		if valveTimer < 10 then
			valve.label:SetBackColor(Turbine.UI.Color(flash,0,0));
		end
	end
end

-- forces valves open by 2 from boss
function forceValvesOpen(valve, gameTime)
	if valve.lastClosed2 < gameTime then
		Turbine.Shell.WriteLine("No close: " .. gameTime .. " - " .. valve.lastClosed2 .. " - " .. valve.lastClosed1);
		estSmokeInCaves = estSmokeInCaves + valve.steps;
		changeValveSteps(valve, 2, gameTime+4);
	elseif valve.lastClosed1 < gameTime then
		Turbine.Shell.WriteLine("One closed: " .. gameTime .. " - " .. valve.lastClosed2 .. " - " .. valve.lastClosed1);
		valve.steps = valve.lastPosition2;
		estSmokeInCaves = estSmokeInCaves + valve.steps;
		changeValveSteps(valve, 2, gameTime);
		changeValveSteps(valve, -1, valve.lastClosed2);
	elseif valve.lastClosed1 > gameTime then
		Turbine.Shell.WriteLine("Both closed: " .. gameTime .. " - " .. valve.lastClosed2 .. " - " .. valve.lastClosed1);
		valve.steps = valve.lastPosition1;
		estSmokeInCaves = estSmokeInCaves + valve.steps;
		changeValveSteps(valve, 2, gameTime);
		changeValveSteps(valve, -1, valve.lastClosed1);
		changeValveSteps(valve, -1, valve.lastClosed1);
	end
end